@@ -36,7 +36,13 @@
 #endif
 #include <netdb.h>
 #ifdef HAVE_PKG_hesiod
+#ifdef __cplusplus
+extern "C" {
+#endif
 #include <hesiod.h>
+#ifdef __cplusplus
+}
+#endif
 #endif
 
 #include <langinfo.h>
@@ -335,19 +341,19 @@ static void send_size_warnings(struct query *ctl)
      */
     if (open_warning_by_mail(ctl))
 	return;
-    stuff_warning(iana_charset, ctl,
-	   GT_("Subject: Fetchmail oversized-messages warning"));
-    stuff_warning(NULL, ctl, "%s", "");
+    stuff_warning(iana_charset, ctl, "Subject: ",
+	   GT_("Fetchmail oversized-messages warning"));
+    stuff_warning(NULL, ctl, "", "%s", "");
     if (ctl->limitflush)
-	stuff_warning(NULL, ctl,
+	stuff_warning(NULL, ctl, "",
 		GT_("The following oversized messages were deleted on server %s account %s:"),
 		ctl->server.pollname, ctl->remotename);
     else
-	stuff_warning(NULL, ctl,
+	stuff_warning(NULL, ctl, "",
 		GT_("The following oversized messages remain on server %s account %s:"),
 		ctl->server.pollname, ctl->remotename);
 
-    stuff_warning(NULL, ctl, "%s", "");
+    stuff_warning(NULL, ctl, "", "%s", "");
 
     if (run.poll_interval == 0)
 	max_warning_poll_count = 0;
@@ -362,12 +368,12 @@ static void send_size_warnings(struct query *ctl)
 	    nbr = current->val.status.mark;
 	    size = atoi(current->id);
 	    if (ctl->limitflush)
-		stuff_warning(NULL, ctl,
+		stuff_warning(NULL, ctl, "",
 			ngettext("  %d message  %d octets long deleted by fetchmail.",
 			         "  %d messages %d octets long deleted by fetchmail.", nbr),
 			nbr, size);
 	    else
-		stuff_warning(NULL, ctl,
+		stuff_warning(NULL, ctl, "",
 			ngettext("  %d message  %d octets long skipped by fetchmail.",
 			         "  %d messages %d octets long skipped by fetchmail.", nbr),
 			nbr, size);
@@ -379,7 +385,7 @@ static void send_size_warnings(struct query *ctl)
 	    current->val.status.num = 0;
     }
 
-    stuff_warning(NULL, ctl, "%s", "");
+    stuff_warning(NULL, ctl, "", "%s", "");
 
     close_warning_by_mail(ctl, (struct msgblk *)NULL);
 }
@@ -432,7 +438,8 @@ static int eat_trailer(int sock, struct query *ctl)
 
 static int fetch_messages(int mailserver_socket, struct query *ctl, 
 			  int count, int **msgsizes, int maxfetch,
-			  int *fetches, int *dispatches, int *deletions)
+			  int *fetches, int *dispatches, int *deletions,
+			  int *transient_errors)
 /* fetch messages in lockstep mode */
 {
     flag force_retrieval;
@@ -505,14 +512,11 @@ static int fetch_messages(int mailserver_socket, struct query *ctl,
         }
 	if (msgcode == MSGLEN_OLD)
 	{
-  		/* To avoid flooding the syslog when using --keep,
-  		 * report "Skipped message" only when:
-  		 *  1) --verbose is on, or
-  		 *  2) fetchmail does not use syslog
-  		 */
-	    if (   (outlevel >= O_VERBOSE) ||
-	           (outlevel > O_SILENT && !run.use_syslog)
-	       )
+	    /*
+	     * To avoid flooding the logs when using --keep, report
+	     * skipping for old messages only when --flush is on.
+	     */
+	    if (outlevel > O_SILENT && ctl->flush)
 	    {
 		report_build(stdout, 
 			     GT_("skipping message %s@%s:%d"),
@@ -538,8 +542,9 @@ static int fetch_messages(int mailserver_socket, struct query *ctl,
 	    lastnum = num + fetchsizelimit - 1;
 	    if (lastnum > count)
 		lastnum = count;
-	    for (i = 0; i < fetchsizelimit; i++)
-		(*msgsizes)[i] = 0;
+	    if (*msgsizes)
+		for (i = 0; i < fetchsizelimit; i++)
+		    (*msgsizes)[i] = 0;
 
 	    stage = STAGE_GETSIZES;
 	    err = (ctl->server.base_protocol->getpartialsizes)(mailserver_socket, num, lastnum, *msgsizes);
@@ -609,6 +614,7 @@ static int fetch_messages(int mailserver_socket, struct query *ctl,
 			     GT_("couldn't fetch headers, message %s@%s:%d (%d octets)\n"),
 			     ctl->remotename, ctl->server.truename, num,
 			     msgsize);
+		(*transient_errors)++;
 		continue;
 	    }
 	    else if (err != 0)
@@ -650,7 +656,10 @@ static int fetch_messages(int mailserver_socket, struct query *ctl,
 	    if (err == PS_RETAINED)
 		suppress_forward = suppress_delete = retained = TRUE;
 	    else if (err == PS_TRANSIENT)
+	    {
 		suppress_delete = suppress_forward = TRUE;
+		(*transient_errors)++;
+	    }
 	    else if (err == PS_REFUSED)
 		suppress_forward = TRUE;
 	    else if (err)
@@ -711,7 +720,10 @@ static int fetch_messages(int mailserver_socket, struct query *ctl,
 			      len);
 
 		if (err == PS_TRANSIENT)
+		{
 		    suppress_delete = suppress_forward = TRUE;
+		    (*transient_errors)++;
+		}
 		else if (err)
 		    return(err);
 
@@ -804,16 +816,11 @@ flagthemail:
 	}
 	else
 	{
-	    if (   (outlevel >= O_VERBOSE) ||
-         		/* To avoid flooding the syslog when using --keep,
-         		 * report "Skipped message" only when:
-         		 *  1) --verbose is on, or
-         		 *  2) fetchmail does not use syslog, or
-         		 *  3) the message was skipped for some other
-         		 *     reason than just being old.
-         		 */
-	           (outlevel > O_SILENT && (!run.use_syslog || msgcode != MSGLEN_OLD))
-	       )
+	    /*
+	     * To avoid flooding the logs when using --keep, report
+	     * skipping of new messages only.
+	     */
+	    if (outlevel > O_SILENT && msgcode != MSGLEN_OLD)
 	    report_complete(stdout, GT_(" not flushed\n"));
 
 	    /* maybe we mark this message as seen now? */
@@ -916,14 +923,14 @@ static int do_session(
 	    if (timeoutcount > MAX_TIMEOUTS 
 		&& !open_warning_by_mail(ctl))
 	    {
-		stuff_warning(iana_charset, ctl,
-			      GT_("Subject: fetchmail sees repeated timeouts"));
-		stuff_warning(NULL, ctl, "%s", "");
-		stuff_warning(NULL, ctl,
+		stuff_warning(iana_charset, ctl, "Subject: ",
+			      GT_("fetchmail sees repeated timeouts"));
+		stuff_warning(NULL, ctl, "", "%s", "");
+		stuff_warning(NULL, ctl, "",
 			      GT_("Fetchmail saw more than %d timeouts while attempting to get mail from %s@%s.\n"), 
 			      MAX_TIMEOUTS,
 			      ctl->remotename, ctl->server.truename);
-		stuff_warning(NULL, ctl, 
+		stuff_warning(NULL, ctl, "",
     GT_("This could mean that your mailserver is stuck, or that your SMTP\n" \
     "server is wedged, or that your mailbox file on the server has been\n" \
     "corrupted by a server error.  You can run `fetchmail -v -v' to\n" \
@@ -942,7 +949,7 @@ static int do_session(
 	/* sigsetjmp returned zero -> normal operation */
 	char buf[MSGBUFSIZE+1], *realhost;
 	int count, newm, bytes;
-	int fetches, dispatches, oldphase;
+	int fetches, dispatches, transient_errors, oldphase;
 	struct idlist *idp;
 
 	/* execute pre-initialization command, if any */
@@ -1077,25 +1084,6 @@ static int do_session(
 		    strlcpy(errbuf, strerror(err_no), sizeof(errbuf));
 		report_complete(stderr, ": %s\n", errbuf);
 
-#ifdef __UNUSED__
-		/* 
-		 * Don't use this.  It was an attempt to address Debian bug
-		 * #47143 (Notify user by mail when pop server nonexistent).
-		 * Trouble is, that doesn't work; you trip over the case 
-		 * where your SLIP or PPP link is down...
-		 */
-		/* warn the system administrator */
-		if (open_warning_by_mail(ctl) == 0)
-		{
-		    stuff_warning(iana_charset, ctl,
-			 GT_("Subject: Fetchmail unreachable-server warning."));
-		    stuff_warning(NULL, ctl, "");
-		    stuff_warning(NULL, ctl, GT_("Fetchmail could not reach the mail server %s:"),
-				  ctl->server.pollname);
-		    stuff_warning(NULL, ctl, errbuf, ctl->server.pollname);
-		    close_warning_by_mail(ctl, (struct msgblk *)NULL);
-		}
-#endif
 	    }
 	    err = PS_SOCKET;
 	    set_timeout(0);
@@ -1119,7 +1107,8 @@ static int do_session(
 		    ctl->sslcommonname : realhost, ctl->server.pollname,
 		    &ctl->remotename) == -1)
 	{
-	    report(stderr, GT_("SSL connection failed.\n"));
+	    set_timeout(0);
+	    report(stderr, "%s: %s", ctl->sslcommonname ? ctl->sslcommonname : realhost, GT_("SSL connection failed.\n"));
 	    err = PS_SOCKET;
 	    goto cleanUp;
 	}
@@ -1209,22 +1198,22 @@ static int do_session(
 			&& !open_warning_by_mail(ctl))
 		    {
 			ctl->wehavesentauthnote = 1;
-			stuff_warning(iana_charset, ctl,
-				      GT_("Subject: fetchmail authentication failed on %s@%s"),
+			stuff_warning(iana_charset, ctl, "Subject: ",
+				      GT_("fetchmail authentication failed on %s@%s"),
 			    ctl->remotename, ctl->server.truename);
-			stuff_warning(NULL, ctl, "%s", "");
-			stuff_warning(NULL, ctl,
+			stuff_warning(NULL, ctl, "", "%s", "");
+			stuff_warning(NULL, ctl, "",
 				      GT_("Fetchmail could not get mail from %s@%s.\n"), 
 				      ctl->remotename,
 				      ctl->server.truename);
 			if (ctl->wehaveauthed) {
-			    stuff_warning(NULL, ctl, GT_("\
+			    stuff_warning(NULL, ctl, "", GT_("\
 The attempt to get authorization failed.\n\
 Since we have already succeeded in getting authorization for this\n\
 connection, this is probably another failure mode (such as busy server)\n\
 that fetchmail cannot distinguish because the server didn't send a useful\n\
 error message."));
-			    stuff_warning(NULL, ctl, GT_("\
+			    stuff_warning(NULL, ctl, "", GT_("\
 \n\
 However, if you HAVE changed your account details since starting the\n\
 fetchmail daemon, you need to stop the daemon, change your configuration\n\
@@ -1234,7 +1223,7 @@ The fetchmail daemon will continue running and attempt to connect\n\
 at each cycle.  No future notifications will be sent until service\n\
 is restored."));
 			} else {
-			    stuff_warning(NULL, ctl, GT_("\
+			    stuff_warning(NULL, ctl, "", GT_("\
 The attempt to get authorization failed.\n\
 This probably means your password is invalid, but some servers have\n\
 other failure modes that fetchmail cannot distinguish from this\n\
@@ -1284,15 +1273,15 @@ is restored."));
 			   ctl->server.truename);
 		    if (!open_warning_by_mail(ctl))
 		    {
-			stuff_warning(iana_charset, ctl,
-			      GT_("Subject: fetchmail authentication OK on %s@%s"), 
+			stuff_warning(iana_charset, ctl, "Subject: ",
+			      GT_("fetchmail authentication OK on %s@%s"), 
 				      ctl->remotename, ctl->server.truename);
-			stuff_warning(NULL, ctl, "%s", "");
-			stuff_warning(NULL, ctl,
+			stuff_warning(NULL, ctl, "", "%s", "");
+			stuff_warning(NULL, ctl, "",
 			      GT_("Fetchmail was able to log into %s@%s.\n"), 
 				      ctl->remotename,
 				      ctl->server.truename);
-			stuff_warning(NULL, ctl, 
+			stuff_warning(NULL, ctl, "",
 				      GT_("Service has been restored.\n"));
 			close_warning_by_mail(ctl, (struct msgblk *)NULL);
 		    
@@ -1317,6 +1306,7 @@ is restored."));
 	    pass = 0;
 	    do {
 		dispatches = 0;
+		transient_errors = 0;
 		++pass;
 
 		/* reset timeout, in case we did an IDLE */
@@ -1446,10 +1436,20 @@ is restored."));
 		    err = fetch_messages(mailserver_socket, ctl, 
 					 count, &msgsizes,
 					 maxfetch,
-					 &fetches, &dispatches, &deletions);
+					 &fetches, &dispatches, &deletions,
+					 &transient_errors);
 		    if (err != PS_SUCCESS && err != PS_MAXFETCH)
 			goto cleanUp;
 
+		    if (transient_errors > MAX_TRANSIENT_ERRORS)
+		    {
+			if (outlevel > O_SILENT)
+			{
+			    report(stderr, GT_("Too many mails skipped (%d > %d) due to transient errors for %s\n"),
+				    transient_errors, MAX_TRANSIENT_ERRORS, buf);
+			}
+		    }
+
 		    if (!check_only && ctl->skipped
 			&& run.poll_interval > 0 && !nodetach)
 		    {
