@@ -29,7 +29,7 @@ import subprocess
 from tkinter import *
 from tkinter.dialog import *
 
-VERSION = "1.63"
+VERSION = "1.63.2"
 
 MIN_PY = (2, 7, 13)
 if sys.version_info < MIN_PY:
@@ -294,7 +294,8 @@ class User(object):
         self.sslcert = None	# SSL certificate filename
         self.sslproto = None	# Force SSL?
         self.sslcertck = True	# Enable strict SSL cert checking
-        self.sslcertpath = None	# Path to trusted certificates
+        self.sslcertfile = None	# Path to bundled file with trusted certificates
+        self.sslcertpath = None	# Path to hashed directory trusted certificates
         self.sslcommonname = None	# SSL CommonName to expect
         self.sslfingerprint = None	# SSL key fingerprint to check
         self.properties = None	# Extension properties
@@ -408,6 +409,8 @@ class User(object):
             res = res + " sslproto " + repr(self.sslproto)
         if self.sslcertck is not None and self.sslcertck != UserDefaults.sslcertck:
             res = res +  flag2str(self.sslcertck, 'sslcertck')
+        if self.sslcertfile and self.sslcertfile != UserDefaults.sslcertfile:
+            res = res + " sslcertfile " + repr(self.sslcertfile)
         if self.sslcertpath and self.sslcertpath != UserDefaults.sslcertpath:
             res = res + " sslcertpath " + repr(self.sslcertpath)
         if self.sslcommonname and self.sslcommonname != UserDefaults.sslcommonname:
@@ -1772,6 +1775,8 @@ class UserEdit(Frame, MyWidget):
                          self.sslcert, '14').pack(side=TOP, fill=X)
             Checkbutton(sslwin, text="Check server SSL certificate?",
                         variable=self.sslcertck).pack(side=TOP, fill=X)
+            LabeledEntry(sslwin, 'SSL trusted certificate bundle file:',
+                         self.sslcertfile, '14').pack(side=TOP, fill=X)
             LabeledEntry(sslwin, 'SSL trusted certificate directory:',
                          self.sslcertpath, '14').pack(side=TOP, fill=X)
             LabeledEntry(sslwin, 'SSL CommonName:',
@@ -2132,10 +2137,6 @@ def copy_instance(toclass, fromdict):
 
 if __name__ == '__main__':
 
-    if "DISPLAY" not in os.environ:
-        print("fetchmailconf must be run under X")
-        sys.exit(1)
-
     fetchmail_icon = """
 R0lGODdhPAAoAPcAAP///wgICBAQEISEhIyMjJSUlKWlpa2trbW1tcbGxs7Ozufn5+/v7//39yEY
 GNa9tUoxKZyEe1o5KTEQAN7OxpyMhIRjUvfn3pxSKYQ5EO/Wxv/WvWtSQrVzSmtCKWspAMatnP/e
@@ -2203,6 +2204,10 @@ welcome to redistribute it under certain conditions.  Please see the file
 COPYING in the source or documentation directory for details.""")
             sys.exit(0)
 
+    if "DISPLAY" not in os.environ:
+        print("fetchmailconf must be run under X")
+        sys.exit(1)
+
     # Get client host's FQDN
     hostname = socket.gethostname()
     if not '.' in hostname:
